// -*- C++ -*-
#include "Rivet/Rivet.hh"
#include "Rivet/RivetAIDA.hh"
#include "Rivet/Particle.hh"
#include "Rivet/ParticleName.hh"
#include "Rivet/Event.hh"
#include "Rivet/Analysis.hh"
#include "Rivet/Projections/FinalState.hh"
#include "Rivet/Projections/ZFinder.hh"
#include <cmath>


namespace Rivet {
 
  /// @
  class LHCB_2012_I1208102 : public Analysis {
  public:
 
    /// Default constructor
    LHCB_2012_I1208102()
      : Analysis("LHCB_2012_I1208102")
    {    }


    /// @name Analysis methods
    //@{

    /// Book histograms
    void init() {
      ZFinder zeefinder(FinalState(), 2.0, 4.5, 20.*GeV, Rivet::ELECTRON, 60.*GeV, 120.*GeV, 0.1, true, true);
      addProjection(zeefinder, "ZeeFinder");
      _nbZEvents = 0.;
      _h_sigma_vs_y   = bookHistogram1D(2, 1, 1);
      _h_sigma_vs_phi = bookHistogram1D(3, 1, 1);
    }



    /// Do the analysis
    void analyze(const Event& e) {
      const double weight = e.weight();
      const ZFinder& zeefinder = applyProjection<ZFinder>(e, "ZeeFinder");
      if (zeefinder.bosons().size() < 1) {
        MSG_INFO("Could not find candidate Z bosons passing cuts in current event...");
        return;
      }

      _nbZEvents += 1.0;

      ///const Particle& zcand = zeefinder.bosons()[0];

      // the following case is not encountered in Rivet 1.x. Pair of leptons closer to Z invariant mass is favoured!!!
      if (zeefinder.bosons().size() > 1) MSG_INFO("Found " << zeefinder.bosons().size() << " decays Z --> e+ e-...");

      // Z momenta:
      const FourMomentum& zee = zeefinder.bosons()[0].momentum();
     
      // Leptons:
      const Particle& pozitron = zeefinder.constituents()[0];
      const Particle& electron = zeefinder.constituents()[1];

      //Calculation of the angular variable:
      double accphi  = PI - Rivet::deltaPhi(pozitron,electron);
      //absolute value taken by default when mapping to 0,PI
      double diffpsd = Rivet::deltaEta(pozitron,electron);
      double angular = tan(accphi/2.)/cosh(diffpsd/2.);

      _h_sigma_vs_y->fill(zee.rapidity(), weight);
      _h_sigma_vs_phi->fill(angular, weight);

    }

    /// Finalize
    void finalize() {
      const double norm = crossSection()/picobarn;
      MSG_WARNING("" << std::endl << "f_Z = " << _nbZEvents/sumOfWeights() << std::endl << "xsection: " << std::scientific << crossSection()/picobarn << " pb");
      normalize(_h_sigma_vs_y, norm);
      normalize(_h_sigma_vs_phi, norm);
    }
    //@}
 
   private:
 
    /// @name Histograms
    //@{
    AIDA::IHistogram1D* _h_sigma_vs_y;
    AIDA::IHistogram1D* _h_sigma_vs_phi;
    double _nbZEvents;

    //@}
 
  };

  // The hook for the plugin system
  DECLARE_RIVET_PLUGIN(LHCB_2012_I1208102);
}

